@@ -3,11 +3,13 @@ package org.gcube.applicationsupportlayer.social.mailing;
 import java.text.Format;
 import java.text.SimpleDateFormat;
 import java.util.Arrays;
+import java.util.Base64;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Set;
 
 import org.gcube.applicationsupportlayer.social.ApplicationNotificationsManager;
+import org.gcube.common.portal.GCubePortalConstants;
 import org.gcube.portal.databook.shared.Comment;
 import org.gcube.portal.databook.shared.Feed;
 import org.gcube.portal.databook.shared.Notification;
@@ -43,7 +45,7 @@ public class SocialMailingUtil {
 	 * @param hashtags
 	 * @return
 	 */
-	protected static String getHTMLEmail(Notification notification2Save, 
+	protected static String getHTMLEmail(String vreName, Notification notification2Save, 
 			String userFirstName, 
 			String portalURL, 
 			String siteLandingPagePath, 
@@ -53,18 +55,17 @@ public class SocialMailingUtil {
 			String commentKey,
 			String ... hashtags) {
 		String removedMarkup = notification2Save.getDescription().replaceAll("&amp;", "&");
-		if (hashtags != null && hashtags.length > 0) {
-			_log.debug("editing hyperlinks for mail client");
-			//notification2Save uri contains the absoulte path to the feed in the correct channel, e.g. /group/ustore_vre?oid=f1637958-34d0-48fc-b5ad-13b1116e389d
-			String pathToVRE = siteLandingPagePath + "?";
-			if (notification2Save.getUri().split("\\?").length > 0) {
-				pathToVRE = notification2Save.getUri().split("\\?")[0];
-			}					
-			removedMarkup = removedMarkup.replace("href=\"?", "href=\""+portalURL + pathToVRE + "?"); //because there is no indication of the portal		
-		}
 
+		//notification2Save uri contains the absoulte path to the feed in the correct channel, e.g. /group/ustore_vre?oid=f1637958-34d0-48fc-b5ad-13b1116e389d		
+		String pathToVRE = siteLandingPagePath + "?";
+		if (notification2Save.getUri().split("\\?").length > 0) {
+			pathToVRE = notification2Save.getUri().split("\\?")[0];
+		}		
+		_log.debug("editing hyperlinks for mail client");
+		removedMarkup = removedMarkup.replaceAll("href=\"/group/", "href=\""+portalURL + "/group/"); //because there is no indication of the portal (for mentions)		
+		removedMarkup = removedMarkup.replace("href=\"?", "href=\""+portalURL + pathToVRE + "?"); //because there is no indication of the portal (for hashtags)		
+			
 		String sender = notification2Save.getSenderFullName();
-
 		String portalHost = portalURL.replaceAll("https://", "");
 		portalHost = portalHost.replaceAll("http://", "");
 
@@ -85,13 +86,15 @@ public class SocialMailingUtil {
 			body.append("<div>").append(WRITE_ABOVE_MESSAGE_REPLY).append("</div><br />");
 			attachmentsNotice = "<br/><p>Please note that email replies do not support attachments.</p>";
 		}
+		
+		String userProfileLink = new StringBuffer(getVREUrl(portalURL, vreName))
+				.append("/").append(getUserProfileLink(notification2Save.getSenderid())).toString();
 
 		body.append("<div style=\"color:#000; font-size:13px; font-family:'lucida grande',tahoma,verdana,arial,sans-serif;\">")
-		.append("Dear ").append(userFirstName).append(",")  //dear <user>
-		.append("<p>").append(sender).append(" ").append(removedMarkup) // has done something
+		.append("<p><a href=\"").append(userProfileLink).append("\">").append(sender).append("</a> ").append(removedMarkup) // has done something
 		.append(getActionLink(notification2Save, portalURL)) //Goto
 		.append(attachmentsNotice)
-		.append(SocialMailingUtil.buildHtmlDiscussion(notification2Save, feed, comments, commentKey)) // the original discussion
+		.append(SocialMailingUtil.buildHtmlDiscussion(portalURL, pathToVRE, notification2Save, feed, comments, commentKey)) // the original discussion
 		.append("</p>")  
 		.append("</div><br/>")
 		.append("<p><div style=\"color:#999999; font-size:11px; font-family:'lucida grande',tahoma,verdana,arial,sans-serif;\">")
@@ -113,6 +116,7 @@ public class SocialMailingUtil {
 
 		return body.toString();
 	}
+
 	/**
 	 * 
 	 * @param notification2Save
@@ -373,6 +377,13 @@ public class SocialMailingUtil {
 	private static StringBuffer buildStringFromNode(Node node) {
 		StringBuffer buffer = new StringBuffer();
 
+		if (node instanceof Element) {
+			Element element = (Element) node;
+			if ("a".equals(element.tagName())) {
+				buffer.append(" ");
+			}
+		}
+		
 		if (node instanceof TextNode) {
 			TextNode textNode = (TextNode) node;
 			buffer.append(textNode.text().trim());
@@ -385,23 +396,25 @@ public class SocialMailingUtil {
 		if (node instanceof Element) {
 			Element element = (Element) node;
 			String tagName = element.tagName();
+			if ("a".equals(element.tagName())) {
+				buffer.append(" ");
+			}
 			if ("p".equals(tagName) || "br".equals(tagName)) {
 				buffer.append("\n");
 			}
 		}
-
 		return buffer;
 	}
 
 	/**
 	 * Build up a discussion given the feed and its comments. 
 	 * @param notification2Save
-	 * @param feed
+	 * @param post
 	 * @param comments
 	 * @param commentKey if not null, when building the discussion stop at this comment.
 	 * @return an html string representing the discussion
 	 */
-	protected static String buildHtmlDiscussion(Notification notification2Save, Feed feed, List<Comment> comments, String commentKey){
+	protected static String buildHtmlDiscussion(String portalURL, String pathToVRE, Notification notification2Save, Feed post, List<Comment> comments, String commentKey){
 
 		try{
 
@@ -415,34 +428,39 @@ public class SocialMailingUtil {
 				Format formatter = new SimpleDateFormat("yyyy-MM-dd hh:mm a");
 
 				// escape html
-				String feedTextNoHtml = convertHTML2Text(feed.getDescription());
-
+				String postTextNoHtml = post.getDescription().replaceAll("&amp;", "&");
+				postTextNoHtml = postTextNoHtml.replaceAll("href=\"/group/", "href=\""+portalURL + "/group/"); //because there is no indication of the portal (for mentions)	
+				postTextNoHtml = postTextNoHtml.replace("href=\"?", "href=\""+portalURL + pathToVRE + "?"); //because there is no indication of the portal	(for hashtags)	
+				
+				
 				// build up html post + comments
 				if(notification2Save.getType() == NotificationType.POST_ALERT || (comments.size() == 0 && notification2Save.getType() == NotificationType.MENTION))
 					htmlPost += "<div style=\"margin-top: 10px;  margin-bottom: 10px;padding-left: 15px;  "
 							+ "font-style: italic; font-weight:bold\">"
-							+ feed.getFullName() 
+							+ post.getFullName() 
 							+ ": " 
-							+ (feedTextNoHtml.length() > 30 ? feedTextNoHtml.substring(0, 30) + " ..." : feedTextNoHtml) 
+							+ (postTextNoHtml.length() > 200 ? postTextNoHtml.substring(0, 200) + " ..." : postTextNoHtml) 
 							+ "<p style=\"font-family:Lucida Grande,"
 							+ "Verdana,Bitstream Vera Sans,Arial,sans-serif; "
-							+ "white-space: nowrap; font-size: smaller; color: #999;\">" + formatter.format(feed.getTime()) + "</p>"
+							+ "white-space: nowrap; font-size: smaller; color: #999;\">" + formatter.format(post.getTime()) + "</p>"
 							+"</div>";
 				else
 					htmlPost += "<div style=\"margin-top: 10px;  margin-bottom: 10px;padding-left: 15px;  "
 							+ "font-style: italic\">"
-							+ feed.getFullName() 
+							+ post.getFullName() 
 							+ ": " 
-							+ (feedTextNoHtml.length() > 30 ? feedTextNoHtml.substring(0, 30) + " ..." : feedTextNoHtml) 
+							+ (postTextNoHtml.length() > 200 ? postTextNoHtml.substring(0, 200) + " ..." : postTextNoHtml) 
 							+ "<p style=\"font-family:Lucida Grande,"
 							+ "Verdana,Bitstream Vera Sans,Arial,sans-serif; "
-							+ "white-space: nowrap; font-size: smaller; color: #999;\">" + formatter.format(feed.getTime()) + "</p>"
+							+ "white-space: nowrap; font-size: smaller; color: #999;\">" + formatter.format(post.getTime()) + "</p>"
 							+"</div>";
 
 				if(comments != null)
 					for (int i = 0; i < comments.size(); i++) {
-
+						
 						String commentTextNoHtml = comments.get(i).getText().replaceAll("&amp;", "&");
+						commentTextNoHtml = commentTextNoHtml.replaceAll("href=\"/group/", "href=\""+portalURL + "/group/"); //because there is no indication of the portal (for mentions)	
+						commentTextNoHtml = commentTextNoHtml.replace("href=\"?", "href=\""+portalURL + pathToVRE + "?"); //because there is no indication of the portal	(for hashtags)	
 
 						if((commentKey != null && comments.get(i).getKey().equals(commentKey)) && !(notification2Save.getType() == NotificationType.POST_ALERT)){
 							htmlPost += "<div style=\"margin-top: 10px;  margin-bottom: 10px;  margin-left: 50px;  padding-left: 15px;  "
@@ -468,7 +486,7 @@ public class SocialMailingUtil {
 									+ "white-space: nowrap; font-size: smaller; color: #999;\">" + formatter.format(comments.get(i).getTime()) + "</p>"
 									+"</div>";
 					} 
-
+				_log.debug("***buildHtmlDiscussion returning=\n"+htmlPost);
 				return htmlPost;
 			}
 		}catch(Exception e){
@@ -538,4 +556,27 @@ public class SocialMailingUtil {
 
 		return "";
 	}
+	/**
+	 * 
+	 * @param gatewayURL
+	 * @param vreName the VRE Name (e.g. AquaMaps)
+	 * @return
+	 */
+	private static String getVREUrl(String gatewayURL, String vreName) {
+		return new StringBuffer(gatewayURL)
+				.append(GCubePortalConstants.PREFIX_GROUP_URL)
+				.append("/").append(vreName.toLowerCase()).toString();
+	}
+	/**
+	 * 
+	 * @param username
+	 * @return
+	 */
+	private static String getUserProfileLink(String username) {
+		return "profile?"+ new String(
+				Base64.getEncoder().encodeToString(GCubePortalConstants.USER_PROFILE_OID.getBytes())+
+				"="+
+						new String( Base64.getEncoder().encodeToString(username.getBytes()) )
+						);
+	}
 }
